#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QDrag>
#include <QMimeData>
#include <QDragEnterEvent>
#include <QMetaProperty>

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    ready = false;

    ui->toolbox->addTopLevelItem (new ButtonFactory);
    ui->toolbox->addTopLevelItem (new EditFactory);
}

MainWindow::~MainWindow()
{
    delete ui;
}

// #include <QDrag>
// #include <QMimeData>


void MainWindow::on_toolbox_itemPressed (QTreeWidgetItem *item, int column)
{
    QDrag * drag = new QDrag (this);
    QMimeData * mimeData = new QMimeData ();
    QByteArray block ((const char*)item, sizeof (item));
    mimeData->setData ("factory", block);
    drag->setMimeData (mimeData);
    drag->exec (Qt::CopyAction | Qt::MoveAction | Qt::LinkAction);
}

void Area::dragEnterEvent (QDragEnterEvent * event)
{
    if (event->mimeData()->hasFormat("factory"))
       event->setAccepted (true);
}

void Area::dropEvent(QDropEvent * event)
{
    const QMimeData * mimeData = event->mimeData();
    if (mimeData->hasFormat("factory"))
    {
        QByteArray block = mimeData->data ("factory");
        Factory * factory =  (Factory  *) block.data ();
        QWidget * widget = factory->create();

        widget->setParent (this);
        widget->setVisible (true);

        QPoint p = event->pos ();
        QRect r = widget->geometry ();
        widget->setGeometry (p.x(), p.y(), r.width(), r.height());

        widget->setObjectName (QString ("new ") + widget->metaObject()->className());
        widget->setProperty ("editable", QVariant (true));
    }

}

void MainWindow::focusChanged (QWidget * old, QWidget * now)
{
    QVariant v = now->property("editable");

    if (! v.isNull())
    {
        ui->info->append (QString ("") + (now ? now->objectName() : "") + " selected ");
        showProperties (now);
    }
}

void MainWindow::showProperties (QWidget * widget)
{
    ready = false;
    ui->table->clear ();

    const QMetaObject * cls = widget->metaObject ();
    int n = cls->propertyCount();
    ui->table->setRowCount (n);
    for (int i = 0; i < n; i++)
    {
        QMetaProperty prop = cls->property (i);
        QTableWidgetItem * item1 = new QTableWidgetItem;
        item1->setText (prop.name());
        ui->table->setItem(i, 0, item1);

        QVariant value = prop.read (widget);

        TableItem * item2 = new TableItem;
        item2->widget = widget;
        item2->prop = prop;
        item2->setText (value.toString ());
        ui->table->setItem (i, 1, item2);
    }
    ready = true;
}

void MainWindow::on_table_itemChanged (QTableWidgetItem * p)
{
    TableItem * item = dynamic_cast <TableItem *> (p);
    if (item != NULL && ready)
    {
        QString s = item->text();
        s = s.trimmed();
        ui->info->append (QString ("") + item->prop.name() + " changed to " + s);

        QWidget * widget = item->widget;
        QMetaProperty prop = item->prop;
        if (prop.type() == QVariant::String)
        {
           QVariant value = s;
           prop.write (widget, value);
        }
        else if (prop.type() == QVariant::Int)
        {
           QVariant value = s.trimmed().toInt();
           prop.write (widget, value);
        }
        else if (prop.type() == QVariant::Bool)
        {
           QVariant value = s.trimmed().toLower() == "true";
           prop.write (widget, value);
        }
    }
}
